using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.Catalog;
using ESRI.ArcGIS.DataManagementTools;
using ESRI.ArcGIS.Display;
using ESRI.ArcGIS.Framework;
using ESRI.ArcGIS.Geoprocessing;
using ESRI.ArcGIS.Geoprocessor;
using ESRI.ArcGIS.esriSystem;
using ESRI.ArcGIS.SystemUI;

namespace CreateWindRendererLayerFile
{
    class Program
    {
        static void Main(string[] args)
        {
            // Variables: Inputs and outputs
            string sInFeature;
            string sFieldWindSpeed;
            string sFieldWindDirection;
            string sOutFeatureClass;
            string sOutFeatureLayerFile;            
            //Additional field name            
            string sFieldWindSpeedT = "WindSpeedT"; //values created from WindSpeed field. The field name is hardcoded.
            // Determine output feature class type
            bool bOutputShapeFile = false;
            bool bOutputGDBFile = false;
            bool bOutputMDBFile = false;
            
            // Create Geoprocessor
            Geoprocessor gp = new Geoprocessor();
            gp.OverwriteOutput = true;

            // Declare GP tools
            AddField addFieldTool = new AddField();            
            CalculateField calculateFieldTool = new CalculateField();
            CopyFeatures copyFeaturesTool = new CopyFeatures();
            Delete deleteTool = new Delete();
            MakeFeatureLayer makeFeatureLayerTool = new MakeFeatureLayer();            
            SaveToLayerFile saveToLayerFileTool = new SaveToLayerFile();

            // Get Arguments               
            // Parameters: TODO
            sInFeature = (string)args.GetValue(0);
            sFieldWindSpeed = (string)args.GetValue(1);
            sFieldWindDirection = (string)args.GetValue(2);
            sOutFeatureClass = (string)args.GetValue(3);
            sOutFeatureLayerFile = (string)args.GetValue(4);
            //sInFeature = @"D:\NetCDFSupport\NetCDFSampleTool\Data\UVToSpeedDirection.shp";
            //sFieldWindSpeed = "Speed";
            //sFieldWindDirection = "Direction";
            //sOutFeatureClass = @"D:\NetCDFSupport\NetCDFSampleTool\Data\UVToSpeedDirectionOut.shp";
            //sOutFeatureLayerFile = @"D:\NetCDFSupport\NetCDFSampleTool\Data\UVToSpeedDirectionOut.lyr";


            // Check output featureclass type
            if (sOutFeatureClass.ToLower().Contains(".shp"))
            {
                bOutputShapeFile = true;
            }
            else if (sOutFeatureClass.ToLower().Contains(".gdb"))
            {
                bOutputGDBFile = true;
            }
            else if (sOutFeatureClass.ToLower().Contains(".mdb"))
            {
                bOutputMDBFile = true;
            }           
            // Check extension in layer file name 
            if (!sOutFeatureLayerFile.ToLower().Contains(".lyr"))
            {
                sOutFeatureLayerFile = sOutFeatureLayerFile + ".lyr";
            }


            // Delete all outputs and intermediate layers if exists        
            DeleteIfExists(gp, deleteTool, sOutFeatureClass); //Delete Output feature class            
            DeleteIfExists(gp, deleteTool, sOutFeatureLayerFile); //Delete output feature layer file
            

            //Step 1. Copy input features to output featureclass
            copyFeaturesTool.in_features = sInFeature;
            copyFeaturesTool.out_feature_class = sOutFeatureClass;
            gp.Execute(copyFeaturesTool, null);
            gp.AddMessage("... Step 1/5: Copied input features to output featureclass");


            //Step 2. Output featureclass: add field WindSpeedT            
            AddFieldUsingGPTool(gp, addFieldTool, sOutFeatureClass, sFieldWindSpeedT, "TEXT", 20);
            gp.AddMessage("... Step 2/5: Added  fields to output featureclass");


            //Step 3. Output featureclass: Populate sFieldWindSpeedT based on sFieldWindSpeed
            string sWindSpeedInMemLayer = "WindSpeedInMemLayer";
            string sSQLExpression = "";
            string sCalcFieldExpression = "";

            //7a. Less than or equal to 2.5
            if (bOutputShapeFile.Equals(true))
            {
                sSQLExpression = Convert.ToChar(34) + sFieldWindSpeed + Convert.ToChar(34) + " <= 2.5";
            }
            else if (bOutputGDBFile.Equals(true))
            {
                sSQLExpression = Convert.ToChar(34) + sFieldWindSpeed + Convert.ToChar(34) + " <= 2.5";
            }
            else if (bOutputMDBFile.Equals(true))
            {
                sSQLExpression = "[" + sFieldWindSpeed + "]" + " <= 2.5";
            }

            DeleteIfExists(gp, deleteTool, sWindSpeedInMemLayer);

            makeFeatureLayerTool.in_features = sOutFeatureClass;
            makeFeatureLayerTool.out_layer = sWindSpeedInMemLayer;
            makeFeatureLayerTool.where_clause = sSQLExpression;
            gp.Execute(makeFeatureLayerTool, null);

            sCalcFieldExpression = Convert.ToChar(34) + "Calm Wind" + Convert.ToChar(34);
            calculateFieldTool.in_table = sWindSpeedInMemLayer;
            calculateFieldTool.field = sFieldWindSpeedT;
            calculateFieldTool.expression = sCalcFieldExpression;
            gp.Execute(calculateFieldTool, null);
            
            // 7b. from 2.5 to 132.5
            double fMinSpeed = 2.5;
            double fMaxSpeed;
            long lAvgSpeed = 5;

            while (fMinSpeed <= 127.5)
            {
                fMaxSpeed = fMinSpeed + 5;

                if (bOutputShapeFile.Equals(true))
                {
                    sSQLExpression = Convert.ToChar(34) + sFieldWindSpeed + Convert.ToChar(34) + " > " + fMinSpeed.ToString() + " AND " + Convert.ToChar(34) + sFieldWindSpeed + Convert.ToChar(34) + " <= " + fMaxSpeed.ToString();
                }
                else if (bOutputGDBFile.Equals(true))
                {
                    sSQLExpression = Convert.ToChar(34) + sFieldWindSpeed + Convert.ToChar(34) + " > " + fMinSpeed.ToString() + " AND " + Convert.ToChar(34) + sFieldWindSpeed + Convert.ToChar(34) + " <= " + fMaxSpeed.ToString();
                }
                else if (bOutputMDBFile.Equals(true))
                {
                    sSQLExpression = "[" + sFieldWindSpeed + "]" + " > " + fMinSpeed.ToString() + " AND " + "[" + sFieldWindSpeed + "]" + " <= " + fMaxSpeed.ToString();
                }

                DeleteIfExists(gp, deleteTool, sWindSpeedInMemLayer);

                makeFeatureLayerTool = new MakeFeatureLayer();
                makeFeatureLayerTool.in_features = sOutFeatureClass;
                makeFeatureLayerTool.out_layer = sWindSpeedInMemLayer;
                makeFeatureLayerTool.where_clause = sSQLExpression;
                gp.Execute(makeFeatureLayerTool, null);

                sCalcFieldExpression = Convert.ToChar(34) + "Wind Speed--" + lAvgSpeed.ToString() + "K" + Convert.ToChar(34);
                calculateFieldTool = new CalculateField();
                calculateFieldTool.in_table = sWindSpeedInMemLayer;
                calculateFieldTool.field = sFieldWindSpeedT;
                calculateFieldTool.expression = sCalcFieldExpression;
                gp.Execute(calculateFieldTool, null);

                fMinSpeed = fMinSpeed + 5;
                lAvgSpeed = lAvgSpeed + 5;
            }

            // 7c. greater than 132.5
            if (bOutputShapeFile.Equals(true))
            {
                sSQLExpression = Convert.ToChar(34) + sFieldWindSpeed + Convert.ToChar(34) + " > 132.5";
            }
            else if (bOutputGDBFile.Equals(true))
            {
                sSQLExpression = Convert.ToChar(34) + sFieldWindSpeed + Convert.ToChar(34) + " > 132.5";
            }
            else if (bOutputMDBFile.Equals(true))
            {
                sSQLExpression = "[" + sFieldWindSpeed + "] > 132.5";
            }

            DeleteIfExists(gp, deleteTool, sWindSpeedInMemLayer);

            makeFeatureLayerTool = new MakeFeatureLayer();
            makeFeatureLayerTool.in_features = sOutFeatureClass;
            makeFeatureLayerTool.out_layer = sWindSpeedInMemLayer;
            makeFeatureLayerTool.where_clause = sSQLExpression;
            gp.Execute(makeFeatureLayerTool, null);

            lAvgSpeed = 135;
            sCalcFieldExpression = Convert.ToChar(34) + "Wind Speed--" + lAvgSpeed.ToString() + "K" + Convert.ToChar(34);
            calculateFieldTool = new CalculateField();
            calculateFieldTool.in_table = sWindSpeedInMemLayer;
            calculateFieldTool.field = sFieldWindSpeedT;
            calculateFieldTool.expression = sCalcFieldExpression;
            gp.Execute(calculateFieldTool, null);
            
            gp.AddMessage("... Step 3/5: Reclassified wind speed in temporary featureclass");


            //Step 4. Create an in-memory layer and then save it to a layer file        
            DeleteIfExists(gp, deleteTool, sWindSpeedInMemLayer);

            makeFeatureLayerTool = new MakeFeatureLayer();
            makeFeatureLayerTool.in_features = sOutFeatureClass;
            makeFeatureLayerTool.out_layer = sWindSpeedInMemLayer;
            gp.Execute(makeFeatureLayerTool, null);

            saveToLayerFileTool.in_layer = sWindSpeedInMemLayer;
            saveToLayerFileTool.out_layer = sOutFeatureLayerFile;
            gp.Execute(saveToLayerFileTool, null);

            gp.AddMessage("... Step 4/5: Created output layer file for output featureclass");


            //======================================================
            //   Assign Unique Value Renderer
            //======================================================

            //Step 5. Set Unique Value Renderer

            //Specify symbols
            IMarkerSymbol pCalmWind = null;
            IMarkerSymbol pWS5K = null, pWS10K = null, pWS15K = null, pWS20K = null, pWS25K = null, pWS30K = null, pWS35K = null, pWS40K = null, pWS45K = null, pWS50K = null, pWS55K = null, pWS60K = null, pWS65K = null, pWS70K = null;
            IMarkerSymbol pWS75K = null, pWS80K = null, pWS85K = null, pWS90K = null, pWS95K = null, pWS100K = null, pWS105K = null, pWS110K = null, pWS115K = null, pWS120K = null, pWS125K = null, pWS130K = null, pWS135K = null;

            IStyleGallery pStyleGallery = new StyleGalleryClass();
            IStyleGalleryStorage pStyleGalleryStorage;
            pStyleGalleryStorage = (IStyleGalleryStorage)pStyleGallery;

            //Add the Weather.style to the IStyleGalleryStorage
            string sWeatherStylePath;
            sWeatherStylePath = pStyleGalleryStorage.DefaultStylePath + "Weather.style";
            pStyleGalleryStorage.AddFile(sWeatherStylePath);

            IEnumStyleGalleryItem pStyleItems;
            pStyleItems = pStyleGallery.get_Items("Marker Symbols", sWeatherStylePath, "");
            pStyleItems.Reset();

            IStyleGalleryItem pStyleItem;
            pStyleItem = pStyleItems.Next();

            while (pStyleItem != null)
            {
                switch (pStyleItem.Name)
                {
                    case "Wind Speed--5K":
                        pWS5K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--10K":
                        pWS10K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--15K":
                        pWS15K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--20K":
                        pWS20K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--25K":
                        pWS25K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--30K":
                        pWS30K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--35K":
                        pWS35K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--40K":
                        pWS40K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--45K":
                        pWS45K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--50K":
                        pWS50K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--55K":
                        pWS55K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--60K":
                        pWS60K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--65K":
                        pWS65K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--70K":
                        pWS70K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--75K":
                        pWS75K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--80K":
                        pWS80K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--85K":
                        pWS85K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--90K":
                        pWS90K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--95K":
                        pWS95K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--100K":
                        pWS100K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--105K":
                        pWS105K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--110K":
                        pWS110K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--115K":
                        pWS115K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--120K":
                        pWS120K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--125K":
                        pWS125K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--130K":
                        pWS130K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                    case "Wind Speed--135K":
                        pWS135K = (IMarkerSymbol)pStyleItem.Item;
                        break;
                }
                pStyleItem = pStyleItems.Next();
            }


            //Add the ESRI.style to the IStyleGalleryStorage
            string sESRIStylePath;
            sESRIStylePath = pStyleGalleryStorage.DefaultStylePath + "ESRI.style";
            pStyleItems = pStyleGallery.get_Items("Marker Symbols", sESRIStylePath, "");
            pStyleItems.Reset();

            pStyleItem = pStyleItems.Next();
            while (pStyleItem != null)
            {
                switch (pStyleItem.Name)
                {
                    case "Circle 4":
                        pCalmWind = (IMarkerSymbol)pStyleItem.Item;            
                        pCalmWind.Size = 8;
                        break;
                }
                pStyleItem = pStyleItems.Next();
            }

            //Specify the unique value renderer
            IUniqueValueRenderer pUVRenderer = new UniqueValueRenderer();
            pUVRenderer.FieldCount = 1;
            pUVRenderer.set_Field(0, sFieldWindSpeedT);
            pUVRenderer.DefaultSymbol = (ISymbol)pCalmWind;
            pUVRenderer.UseDefaultSymbol = false;

            //Adding values
            pUVRenderer.AddValue("Calm Wind", "", (ISymbol)pCalmWind);
            pUVRenderer.AddValue("Wind Speed--5K", "", (ISymbol)pWS5K);
            pUVRenderer.AddValue("Wind Speed--10K", "", (ISymbol)pWS10K);
            pUVRenderer.AddValue("Wind Speed--15K", "", (ISymbol)pWS15K);
            pUVRenderer.AddValue("Wind Speed--20K", "", (ISymbol)pWS20K);
            pUVRenderer.AddValue("Wind Speed--25K", "", (ISymbol)pWS25K);
            pUVRenderer.AddValue("Wind Speed--30K", "", (ISymbol)pWS30K);
            pUVRenderer.AddValue("Wind Speed--35K", "", (ISymbol)pWS35K);
            pUVRenderer.AddValue("Wind Speed--40K", "", (ISymbol)pWS40K);
            pUVRenderer.AddValue("Wind Speed--45K", "", (ISymbol)pWS45K);
            pUVRenderer.AddValue("Wind Speed--50K", "", (ISymbol)pWS50K);
            pUVRenderer.AddValue("Wind Speed--55K", "", (ISymbol)pWS55K);
            pUVRenderer.AddValue("Wind Speed--60K", "", (ISymbol)pWS60K);
            pUVRenderer.AddValue("Wind Speed--65K", "", (ISymbol)pWS65K);
            pUVRenderer.AddValue("Wind Speed--70K", "", (ISymbol)pWS70K);
            pUVRenderer.AddValue("Wind Speed--75K", "", (ISymbol)pWS75K);
            pUVRenderer.AddValue("Wind Speed--80K", "", (ISymbol)pWS80K);
            pUVRenderer.AddValue("Wind Speed--85K", "", (ISymbol)pWS85K);
            pUVRenderer.AddValue("Wind Speed--90K", "", (ISymbol)pWS90K);
            pUVRenderer.AddValue("Wind Speed--95K", "", (ISymbol)pWS95K);
            pUVRenderer.AddValue("Wind Speed--100K", "", (ISymbol)pWS100K);
            pUVRenderer.AddValue("Wind Speed--105K", "", (ISymbol)pWS105K);
            pUVRenderer.AddValue("Wind Speed--110K", "", (ISymbol)pWS110K);
            pUVRenderer.AddValue("Wind Speed--115K", "", (ISymbol)pWS115K);
            pUVRenderer.AddValue("Wind Speed--120K", "", (ISymbol)pWS120K);
            pUVRenderer.AddValue("Wind Speed--125K", "", (ISymbol)pWS125K);
            pUVRenderer.AddValue("Wind Speed--130K", "", (ISymbol)pWS130K);
            pUVRenderer.AddValue("Wind Speed--135K", "", (ISymbol)pWS135K);

            //Specify rotation
            IRotationRenderer pRotationRenderer;
            pRotationRenderer = (IRotationRenderer)pUVRenderer;
            pRotationRenderer.RotationField = sFieldWindDirection; //Field Name
            pRotationRenderer.RotationType = esriSymbolRotationType.esriRotateSymbolGeographic;

            //Get the layer file
            int lNum;
            object vValue;
            IEnumGxObject pEnumGxObject;
            IGxCatalog pGxCat = new GxCatalogClass();
            IGxObject pGxObj;

            vValue = pGxCat.GetObjectFromFullName(sOutFeatureLayerFile, out lNum);

            //if (vValue.GetType() is IEnumGxObject)
            if (lNum > 1)
            {
                pEnumGxObject = (IEnumGxObject)vValue;
                pGxObj = pEnumGxObject.Next();
            }
            else
            {
                pGxObj = (IGxObject)vValue;
            }
            
            IGxLayer pGxLayer;
            IGeoFeatureLayer pGeoFLayer;
            pGxLayer = (IGxLayer)pGxObj;
            pGeoFLayer = (IGeoFeatureLayer)pGxLayer.Layer;

            //Update the GeoFeatureLayer and the GxLayer
            pGeoFLayer.Renderer = (IFeatureRenderer)pUVRenderer;
            pGeoFLayer.Name = "WindVelocity";
            pGxLayer.Layer = (ILayer)pGeoFLayer;
            gp.AddMessage("... Step 5/5: Assigned renderer to output layer file");
        }

        private static void AddFieldUsingGPTool(Geoprocessor gp, AddField addFieldTool, string sInTable, string sFieldName, string sFieldType, int lFieldLength)
        {
            addFieldTool = new AddField();
            addFieldTool.in_table = sInTable;
            addFieldTool.field_name = sFieldName;
            addFieldTool.field_type = sFieldType;
            if (sFieldType.ToUpper().Equals("LONG"))
            {
                addFieldTool.field_precision = lFieldLength;
            }
            else if (sFieldType.ToUpper().Equals("TEXT"))
            {
                addFieldTool.field_length = lFieldLength;
            }
            gp.Execute(addFieldTool, null);
        }
        private static void DeleteIfExists(Geoprocessor gp, Delete deleteTool, string sName)
        {
            object x = null;
            if (gp.Exists(sName, ref x))
            {
                deleteTool.in_data = sName;
                gp.Execute(deleteTool, null);
            }

        }
    }
}
